@@ -19,13 +19,15 @@ under the License.
 
 import re
 import os
+import stat
+import json
 
 from node import CFSNode
 from target import Target
 from fabric import FabricModule
 from tcm import (StorageObject, FileIOStorageObject, BlockStorageObject,
                  PSCSIStorageObject, RDMCPStorageObject)
-from utils import RTSLibError, RTSLibBrokenLink, modprobe
+from utils import RTSLibError, RTSLibBrokenLink, modprobe, mount_configfs
 from utils import dict_remove, set_attributes
 
 storageobjects = dict(
@@ -35,6 +37,8 @@ storageobjects = dict(
     ramdisk=RDMCPStorageObject,
     )
 
+default_save_file = "/etc/target/saveconfig.json"
+
 class RTSRoot(CFSNode):
     '''
     This is an interface to the root of the configFS object tree.
@@ -66,6 +70,8 @@ class RTSRoot(CFSNode):
         base kernel modules (tcm)
         '''
         super(RTSRoot, self).__init__()
+        modprobe('configfs')
+        mount_configfs()
         modprobe('target_core_mod')
         self._create_in_cfs_ine('any')
 
@@ -149,7 +155,7 @@ class RTSRoot(CFSNode):
         for so in self.storage_objects:
             so.delete()
 
-    def restore(self, config, clear_existing=False):
+    def restore(self, config, clear_existing=False, abort_on_error=False):
         '''
         Takes a dict generated by dump() and reconfigures the target to match.
         Returns list of non-fatal errors that were encountered.
@@ -162,8 +168,12 @@ class RTSRoot(CFSNode):
 
         errors = []
 
-        def err_func(err_str):
-            errors.append(err_str + ", skipped")
+        if abort_on_error:
+            def err_func(err_str):
+                raise RTSLibError(err_str)
+        else:
+            def err_func(err_str):
+                errors.append(err_str + ", skipped")
 
         for index, so in enumerate(config.get('storage_objects', [])):
             if 'name' not in so:
@@ -214,6 +224,35 @@ class RTSRoot(CFSNode):
 
         return errors
 
+    def save_to_file(self, save_file=None):
+        '''
+        Write the configuration in json format to a file.
+        '''
+        if not save_file:
+            save_file = default_save_file
+
+        with open(save_file+".temp", "w+") as f:
+            os.fchmod(f.fileno(), stat.S_IRUSR | stat.S_IWUSR)
+            f.write(json.dumps(self.dump(), sort_keys=True, indent=2))
+            f.write("\n")
+            os.fsync(f.fileno())
+
+        os.rename(save_file+".temp", save_file)
+
+    def restore_from_file(self, restore_file=None, clear_existing=True, abort_on_error=False):
+        '''
+        Restore the configuration from a file in json format.
+        Returns a list of non-fatal errors. If abort_on_error is set,
+          it will raise the exception instead of continuing.
+        '''
+        if not restore_file:
+            restore_file = default_save_file
+
+        with open(restore_file, "r") as f:
+            config = json.loads(f.read())
+            return self.restore(config, clear_existing=clear_existing,
+                                abort_on_error=abort_on_error)
+
     targets = property(_list_targets,
             doc="Get the list of Target objects.")
     tpgs = property(_list_tpgs,
