@@ -25,7 +25,7 @@ import uuid
 
 from node import CFSNode
 from os.path import isdir
-from utils import RTSLibError, RTSLibBrokenLink, modprobe
+from utils import RTSLibError, RTSLibBrokenLink
 from utils import fread, fwrite, normalize_wwn, generate_wwn
 from utils import dict_remove, set_attributes, set_parameters, ignored
 from utils import _get_auth_attr, _set_auth_attr
@@ -238,7 +238,7 @@ class TPG(CFSNode):
         if os.path.isfile(path) and (boolean != self._get_enable()):
             try:
                 fwrite(path, str(int(boolean)))
-            except IOError, e:
+            except IOError as e:
                 raise RTSLibError("Cannot change enable state: %s" % e)
 
     def _get_nexus(self):
@@ -404,7 +404,6 @@ class TPG(CFSNode):
     @classmethod
     def setup(cls, t_obj, tpg, err_func):
         tpg_obj = cls(t_obj, tag=tpg.get("tag", None))
-        tpg_obj.enable = tpg.get('enable', True)
         set_attributes(tpg_obj, tpg.get('attributes', {}))
         set_parameters(tpg_obj, tpg.get('parameters', {}))
 
@@ -417,6 +416,7 @@ class TPG(CFSNode):
         for acl in tpg.get('node_acls', []):
             NodeACL.setup(tpg_obj, acl, err_func)
 
+        tpg_obj.enable = tpg.get('enable', True)
         dict_remove(tpg, ('luns', 'portals', 'node_acls', 'tag',
                           'attributes', 'parameters', 'enable'))
         for name, value in tpg.iteritems():
@@ -448,7 +448,7 @@ class LUN(CFSNode):
     A LUN is identified by its parent TPG and LUN index.
     '''
 
-    MAX_LUN = 255
+    MAX_LUN = 16383
 
     # LUN private stuff
 
@@ -696,7 +696,7 @@ class NetworkPortal(CFSNode):
 
         try:
             self._create_in_cfs_ine(mode)
-        except OSError, msg:
+        except OSError as msg:
             raise RTSLibError(msg[1])
 
     def _get_ip_address(self):
@@ -818,7 +818,7 @@ class NodeACL(CFSNode):
         path = "%s/cmdsn_depth" % self.path
         try:
             fwrite(path, "%s" % depth)
-        except IOError, msg:
+        except IOError as msg:
             msg = msg[1]
             raise RTSLibError("Cannot set tcq_depth: %s" % str(msg))
 
