@@ -111,12 +111,20 @@ def is_dev_in_use(path):
 
 def get_blockdev_size(path):
     '''
-    Returns the size in blocks of a disk-type block device.
+    Returns the size in logical blocks of a disk-type block device.
     '''
     name = os.path.basename(os.path.realpath(path))
 
+    # size is in 512-byte sectors, we want to return number of logical blocks
+    def get_size(path, is_partition=False):
+        sect_size = int(fread("%s/size" % path))
+        if is_partition:
+            path = os.path.split(path)[0]
+        logical_block_size = int(fread("%s/queue/logical_block_size" % path))
+        return sect_size / (logical_block_size / 512)
+
     try:
-        return int(fread("/sys/block/%s/size" % name))
+        return get_size("/sys/block/%s" % name)
     except IOError:
         # Maybe it's a partition?
         m = re.search(r'^([a-z0-9_-]+)(\d+)$', name)
@@ -126,7 +134,7 @@ def get_blockdev_size(path):
             disk = m.groups()[0]
             if disk[-1] == 'p' and disk[-2].isdigit():
                 disk = disk[:-1]
-            return int(fread("/sys/block/%s/%s/size" % (disk, m.group())))
+            return get_size("/sys/block/%s/%s" % (disk, m.group()), True)
         else:
             raise
 
@@ -262,7 +270,7 @@ def generate_wwn(wwn_type):
         return str(uuid.uuid4())
     elif wwn_type == 'iqn':
         localname = socket.gethostname().split(".")[0]
-        localarch = os.uname()[4].replace("_","")
+        localarch = os.uname()[4].replace("_", "")
         prefix = "iqn.2003-01.org.linux-iscsi.%s.%s" % (localname, localarch)
         prefix = prefix.strip().lower()
         serial = "sn.%s" % str(uuid.uuid4())[24:]
@@ -294,7 +302,7 @@ def _cleanse_wwn(wwn_type, wwn):
     if wwn_type in ('naa', 'eui', 'ib'):
         if wwn.startswith("0x"):
             wwn = wwn[2:]
-        wwn = wwn.translate(None, ":-")
+        wwn = wwn.replace(":-", "")
 
         if not (wwn.startswith("naa.") or wwn.startswith("eui.") or \
             wwn.startswith("ib.")):
@@ -311,15 +319,15 @@ def normalize_wwn(wwn_types, wwn, possible_wwns=None):
     Returns (normalized_wwn, wwn_type), or exception if invalid wwn.
     '''
     wwn_test = {
-    'free' : lambda wwn: True,
-    'iqn' : lambda wwn: \
+    'free': lambda wwn: True,
+    'iqn': lambda wwn: \
         re.match("iqn\.[0-9]{4}-[0-1][0-9]\..*\..*", wwn) \
         and not re.search(' ', wwn) \
         and not re.search('_', wwn),
-    'naa' : lambda wwn: re.match("naa\.[125][0-9a-fA-F]{15}$", wwn),
-    'eui' : lambda wwn: re.match("eui\.[0-9a-f]{16}$", wwn),
-    'ib' : lambda wwn: re.match("ib\.[0-9a-f]{32}$", wwn),
-    'unit_serial' : lambda wwn: \
+    'naa': lambda wwn: re.match("naa\.[125][0-9a-fA-F]{15}$", wwn),
+    'eui': lambda wwn: re.match("eui\.[0-9a-f]{16}$", wwn),
+    'ib': lambda wwn: re.match("ib\.[0-9a-f]{32}$", wwn),
+    'unit_serial': lambda wwn: \
         re.match("[0-9A-Fa-f]{8}(-[0-9A-Fa-f]{4}){3}-[0-9A-Fa-f]{12}$", wwn),
     }
 
@@ -363,6 +371,16 @@ def modprobe(module):
         if process.returncode != 0:
             raise RTSLibError(stderrdata)
 
+def mount_configfs():
+    if not os.path.ismount("/sys/kernel/config"):
+        cmdline = "mount -t configfs none /sys/kernel/config"
+        process = subprocess.Popen(cmdline.split(),
+                                   stdout=subprocess.PIPE,
+                                   stderr=subprocess.PIPE)
+        (stdoutdata, stderrdata) = process.communicate()
+        if process.returncode != 0:
+            raise RTSLibError("Cannot mount configfs.")
+
 def dict_remove(d, items):
     for item in items:
         if item in d:
